module ChkSumChk(Zero, Ld1, Ld2, Ld3, Ld4, LdChk, ChkOK, PktRcvd, OP1, OP0, Load, Clk, Reset);
  input		Zero;
  input		Ld1;
  input		Ld2;
  input		Ld3;
  input		Ld4;
  input		LdChk;
  output	ChkOK;
  output	PktRcvd;
  output	OP1;
  output	OP0;
  output	Load;
  input		Clk;
  input		Reset;

  reg 		OP0, OP1; 
  reg		Load;
  reg		ChkOK;
  reg		PktRcvd;

  /* Internals */
  reg [2:0]	state, next_state;

  /* State encoding */
  `define Start   3'b000 /* start state                                                 */
  `define Done1	  3'b001 /* seen first nibble, wait for second                          */
  `define Done2   3'b010 /* seen two nibbles, wait for third                            */
  `define Done3   3'b011 /* seen three nibbles, wait for fourth                         */
  `define Done4   3'b100 /* seen all four nibbles, wait for chksum                      */
  `define DoneChk 3'b111 /* seen chksum nibble, packet completely received, ChkOK valid */

  /* State flips-flops with reset and ChkOK flip-flop */
  always @(posedge Clk) begin
	if (Reset) state = `Start;
	else state = next_state;
	ChkOK = (Zero & Load & LdChk);
  end

  /* Next state logic with ALU and chksum register control outputs */
  always @(Ld1 or Ld2 or Ld3 or Ld4 or LdChk or state) begin 
	case (state)
	`Start:   if (Ld1)   begin OP0 = 0; OP1 = 1; Load = 1; next_state = `Done1;   end
		  else	     begin                   Load = 0; next_state = `Start;   end
	`Done1:   if (Ld2)   begin OP0 = 0; OP1 = 0; Load = 1; next_state = `Done2;   end
		  else	     begin                   Load = 0; next_state = `Done1;   end
	`Done2:   if (Ld3)   begin OP0 = 0; OP1 = 0; Load = 1; next_state = `Done3;   end
		  else	     begin                   Load = 0; next_state = `Done2;   end
	`Done3:   if (Ld4)   begin OP0 = 0; OP1 = 0; Load = 1; next_state = `Done4;   end
		  else	     begin                   Load = 0; next_state = `Done3;   end
	`Done4:   if (LdChk) begin OP0 = 1; OP1 = 0; Load = 1; next_state = `DoneChk; end
		  else	     begin                   Load = 0; next_state = `Done4;   end
	`DoneChk:            begin                   Load = 1; next_state = `Start;   end
	endcase
  end

  /* Output logic */
  assign PktRcvd = (state == `DoneChk); 

endmodule
