module Deframe(Sin, Sout, Pval, Clk, Reset);
  input		Sin;
  output	Sout;
  output	Pval;
  input		Clk;
  input		Reset;

  reg		Sout;
  reg		Pval;

  /* Internals */
  reg [4:0] 	last5;
  reg [1:0]	count; 
  reg		packet;

  /* State flip-flops, next state logic including reset, and output logic */
  always @(posedge Clk) begin
	if (Reset) begin
		Pval   = 0;
		count  = 0;
		packet = 0;
	end
	else begin
		// look for start marker
		if (packet == 0) begin 
			if (last5[3:0] == 4'b1110) begin
				packet = 1;
			end
		end
		else begin
			// packet data has reached end of shift register
			if (~Pval & (count == 3)) begin
				Pval  = 1; 
			end

			// increment count to wait for packet data 
			// to reach end of shift register
			if (~Pval) begin
				count = count + 1;
			end

			// look for end marker
			if (last5[3:0] == 4'b1110) begin
				Pval   = 0;
				count  = 0;
				packet = 0;
			end
		end
	end
	last5 = {last5[3:0], Sin};  // data shift register
  end

  /* Data output */
  assign Sout = last5[4];

endmodule
