module Destuff(Sin, Pval, PktRcvd, LenOK, Sout, Sval, Clk, Reset);
  input		Sin;
  input		Pval;
  input		PktRcvd;
  output	LenOK;
  output	Sout;
  output	Sval;
  input		Clk;
  input		Reset;

  reg		Sval;
  reg		Sout;
  reg		LenOK;

  /* Internals */
  reg [3:0] state, next_state; 
  reg [4:0] count, next_count;

  /* State encoding extra bit + {Sout, Sval, LenOK} */ 
  `define Start 4'b0000 /* start state              */
  `define Zero	4'b0010 /* saw a zero               */
  `define One	4'b0110 /* saw a first one          */
  `define Two	4'b1110 /* saw a second one         */
  `define Skip	4'b1000 /* skip over stuffed zero   */
  `define Done  4'b0001 /* hold LenOK high          */

  /* State and count flip-flops with reset */
  always @(posedge Clk) begin 
	if(Reset) begin 
		state    = `Start; 
		count    = 5'b00000;
	end 
	else begin 
		state    = next_state; 
		count    = next_count;
	end 
  end 

  /* Next state logic */
  always @(Sin or Pval or PktRcvd or count or state) begin 
	if (Pval) begin 
		case (state)
		`Start: begin 
			if (Sin)     next_state = `One; else next_state = `Zero;
			next_count = count + 5'b00001;
		end
		`Zero:  begin 
			if (Sin)     next_state = `One; else next_state = `Zero; 
			next_count = count + 5'b00001;
		end 
		`One:   begin 
			if (Sin)     next_state = `Two; else next_state = `Zero; 
			next_count = count + 5'b00001;
		end 
		`Two:   begin
			             next_state = `Skip; 
		end
		`Skip:  begin 
			if (Sin)     next_state = `One; else next_state = `Zero; 
			next_count = count + 5'b00001;
		end 
		`Done:	begin
			if (PktRcvd) next_state = `Start; else next_state = `Done;
		end
		endcase
	end 
	else begin 
		case (state)
		`Start: begin
			next_state = `Start;
		end
		`Zero:  begin
			if (count == 20) next_state = `Done;  else next_state = `Start;
		end
		`One:   begin
			if (count == 20) next_state = `Done;  else next_state = `Start;
		end
		`Two:   begin
			if (count == 20) next_state = `Done;  else next_state = `Start;
		end
		`Skip:  begin
			if (count == 20) next_state = `Done;  else next_state = `Start;
		end
		`Done:	begin
			if (PktRcvd)     next_state = `Start; else next_state = `Done;
		end
		endcase
		next_count = 5'b00000;
	end 
  end 

  /* Output logic */
  assign Sout  = state[2]; 
  assign Sval  = state[1];
  assign LenOK = state[0];

endmodule
