module Start(Start, LoadPkt, E, Sending, Clk, Reset);
  input		Start;
  output	LoadPkt;
  output	E;
  input		Sending;
  input		Clk;
  input		Reset;

  reg		LoadPkt;
  reg		E;

  /* Internals */
  reg [1:0] state;
  reg [1:0] next_state;

  /* State encoding */
  `define STARTUP 2'b00 /* wait for start                         */
  `define ENABLE  2'b01 /* signal enable to following blocks      */
  `define WAIT	  2'b10	/* wait until framer block starts sending */
  `define SENDING 2'b11 /* wait until framer block stops sending  */

  /* State flip-flops with reset */
  always @(posedge Clk) begin
 	if (Reset) begin
 		state = `STARTUP;
 	end
 	else begin
 		state = next_state;
 	end
  end

  /* Next state logic */
  always @(Start or Sending or state) begin
 	case(state)
 	 `STARTUP: begin
 		if (Start) next_state = `ENABLE;
 		else next_state = `STARTUP;
 	 end
	 `ENABLE: begin
 		next_state = `WAIT;
  	 end
	 `WAIT: begin
 		if (Sending) next_state = `SENDING;
 		else next_state = `WAIT;
 	 end
	 `SENDING: begin
		if (Sending) next_state = `SENDING;
		else next_state = `STARTUP;
	 end
 	endcase
  end

  /* Output logic */
  always @(state) begin
	case (state)
	 `STARTUP: begin
		LoadPkt = 1; E = 0; /* load value on packet keyboards                  */
	 end
	 `ENABLE:  begin
		LoadPkt = 0; E = 1; /* enable next block and wait for sending to start */
	 end
	 `WAIT:  begin
		LoadPkt = 0; E = 1; /* enable next block and wait for sending to start */
	 end
	 `SENDING: begin
		LoadPkt = 0; E = 1; /* wait for sending to stop                        */
	 end
	endcase
  end

endmodule
